from datetime import datetime
from typing import List
import pandas as pd
from backtest.lib import AnalysisResult
from backtest.lib import BacktestCandidate
from scanner.interfaces import ScannerInterface
from backtest.interfaces import AnalyzerInterface


class BasicBacktest:
    def __init__(self, scanner: ScannerInterface, from_date: datetime, to_date: datetime,
                 analyzers: List[AnalyzerInterface]):
        self.scanner: ScannerInterface = scanner
        self.from_date = from_date
        self.to_date = to_date
        self.analyzers = analyzers

    def load_data(self, verbose: bool = False) -> "BasicBacktest":
        self.scanner.load_data(verbose)
        return self

    def run(self):
        dates = pd.date_range(start=self.from_date, end=self.to_date).tolist()
        candidates = []
        for current_datetime in dates:
            current_candidates = self.scanner.get_candidates(current_datetime, verbose=False)

            for item in current_candidates:
                analyses: List[AnalysisResult] = []
                for analyzer in self.analyzers:
                    try:
                        analyses.append(analyzer.get_result(item, current_datetime))
                    except (KeyError, IndexError):
                        continue
                candidate: BacktestCandidate = BacktestCandidate(item, current_datetime, analyses)
                candidates.append(candidate)
        return candidates
