from datetime import datetime

from backtest.lib import AnalysisResult
from backtest.interfaces import AnalyzerInterface
from stock.interfaces import StockInterface


class PerformanceNDays(AnalyzerInterface):
    def __init__(self, period: int = 30):
        self.period: int = period
        self.description: str = "The relative change of the stock's close price in 30 days since it showed up on scan."

    def get_result(self, stock:  StockInterface, in_scan_date: datetime) -> AnalysisResult:
        current_date: str = in_scan_date.strftime("%Y-%m-%d")
        current_date_price: float = stock.get_close().loc[current_date]
        next_month_price: float = stock.get_close().loc[current_date:][self.period]
        performance: float = ((next_month_price - current_date_price) / current_date_price)
        return AnalysisResult(performance, self.description)
